@@ -13,7 +13,7 @@ no warnings qw( threads recursion uninitialized numeric once );
 
 package MCE::Shared::Server;
 
-our $VERSION = '1.881';
+our $VERSION = '1.882';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -160,7 +160,7 @@ sub _new {
 
       for my $_k (qw(
          _qw_sock _qr_sock _aw_sock _ar_sock _cw_sock _cr_sock
-         _mutex_0 _mutex_1 _mutex_2 _mutex_3 _mutex_4 _mutex_5
+         _mutex_0 _mutex_1 _mutex_2 _mutex_3 _mutex_4 _mutex_5 _qr_mutex
       )) {
          if ( defined $_[1]->{ $_k } ) {
             $_hndls{ $_k } = delete $_[1]->{ $_k };
@@ -1303,7 +1303,6 @@ sub _start {
    # inlined for performance
    $_dat_ex = sub {
       my $_pid = $_tid ? $$ .'.'. $_tid : $$;
-      MCE::Util::_sock_ready($_DAT_LOCK->{_r_sock}) if $_is_MSWin32;
       MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
          unless $_DAT_LOCK->{ $_pid };
    };
@@ -1445,8 +1444,8 @@ sub _get_hobo_data {
    return ($_result, $_error);
 }
 
-# Called by await, rewind, broadcast, signal, timedwait, and wait.
-# Including CLOSE, DESTROY, and destroy.
+# Called by await, dequeue_timed, rewind, broadcast, signal, timedwait, and
+# wait. Including CLOSE, DESTROY, and destroy.
 
 sub _req1 {
    return unless defined $_DAU_W_SOCK;  # (in cleanup)
@@ -1933,7 +1932,7 @@ MCE::Shared::Server - Server/Object packages for MCE::Shared
 
 =head1 VERSION
 
-This document describes MCE::Shared::Server version 1.881
+This document describes MCE::Shared::Server version 1.882
 
 =head1 DESCRIPTION
 
